use nix::{
    sys::signal::{signal, SigHandler, Signal},
    unistd::{chdir, close as fdclose, fork, getppid, setsid, ForkResult},
};
use std::process::exit;

pub trait OneOrVec<T> {
    fn into_vec(self) -> Vec<T>;
}

impl<T> OneOrVec<T> for T {
    fn into_vec(self) -> Vec<T> {
        vec![self]
    }
}

impl<T> OneOrVec<T> for Vec<T> {
    fn into_vec(self) -> Vec<T> {
        self
    }
}

#[macro_export]
macro_rules! unwrap_enum {
    ($src:ident, $t:path) => {
        if let $t(result) = $src {
            result
        } else {
            panic!("wrong type")
        }
    };
}

pub fn daemonize() {
    if getppid().as_raw() != 1 {
        setsig(Signal::SIGTTOU, SigHandler::SigIgn);
        setsig(Signal::SIGTTIN, SigHandler::SigIgn);
        setsig(Signal::SIGTSTP, SigHandler::SigIgn);
    }
    for fd in 0..=2 {
        match fdclose(fd) {
            _ => (),
        }
    }
    match chdir("/") {
        _ => (),
    };

    match fork() {
        Ok(ForkResult::Parent { .. }) => {
            exit(0);
        }
        Ok(ForkResult::Child) => match setsid() {
            _ => (),
        },
        Err(_) => exit(255),
    }
}

pub fn setsig(sig: Signal, hnd: SigHandler) {
    unsafe {
        signal(sig, hnd).unwrap();
    }
}

pub fn init_env() {
    let envs = [".env"];
    for envfile in &envs {
        dotenv::from_filename(envfile).ok();
    }
}
